import os
import logging
import click
import sys

from vcftoolbox import get_vcf_handle

from loqusdb.exceptions import CaseError
from loqusdb.vcf_tools import get_formated_variant
from loqusdb.utils import get_family
from loqusdb.utils import load_variants

from . import base_command

logger = logging.getLogger(__name__)


@base_command.command()
@click.argument('variant_file',
                    nargs=1,
                    type=click.Path(),
                    metavar='<vcf_file> or -'
)
@click.option('-f', '--family_file',
                    nargs=1, 
                    type=click.File('r'),
                    metavar='<ped_file>'
)
@click.option('-t' ,'--family_type', 
                type=click.Choice(['ped', 'alt', 'cmms', 'mip']), 
                default='ped',
                help='If the analysis use one of the known setups, please specify which one.'
)
@click.option('-b' ,'--bulk_insert', 
                is_flag=True, 
                help='Insert bulks of variants for better performance'
)
@click.pass_context
def load(ctx, variant_file, family_file, family_type, bulk_insert):
    """Load the variants of a case

    The loading is based on if the variant is seen in a ny affected individual
    in the family.
    """
    if not family_file:
        logger.error("Please provide a family file")
        ctx.abort()

    if variant_file == '-':
        logger.info("Parsing variants from stdin")
        variant_file = get_vcf_handle(fsock=sys.stdin)
    else:
        logger.info("Start parsing variants from stdin")
        variant_path = os.path.abspath(variant_file)
        variant_file = get_vcf_handle(infile=variant_file)

    try:
        family = get_family(family_lines=family_file, family_type=family_type)
    except SyntaxError as error:
        logger.warning(error.message)
        ctx.abort()

    if not family.affected_individuals:
        logger.error("No affected individuals could be found in ped file")
        ctx.abort()
    logger.info("Found affected individuals in ped file: {0}"
                .format(', '.join(family.affected_individuals)))

    adapter = ctx.obj['adapter']
    try:
        load_variants(adapter, family.family_id, family.affected_individuals,
                      variant_file, bulk_insert=bulk_insert,
                      vcf_path=variant_path)
    except CaseError as error:
        logger.error(error.message)
        ctx.abort()



import logging

from loqusdb.plugins import BaseVariantMixin

logger = logging.getLogger(__name__)

class VariantMixin(BaseVariantMixin):
    
    def add_variant(self, variant):
        """Add a variant to the variant collection
        
            If the variant exists we update the count else we insert a new variant
            object.
        
            Args:
                variant (dict): A variant dictionary
        
        """
        logger.debug("Upserting variant: {0}".format(variant.get('_id')))
    
        message = self.db.variant.update(
            {'_id': variant['_id']},
            {
                '$inc': {
                    'homozygote': variant.get('homozygote', 0),
                    'observations': 1
                }
             }, 
             upsert=True
        )
    
        if message.get('updatedExisting'):
            logger.debug("Variant {0} was updated".format(message.get("upserted")))
        else:
            logger.debug("Variant was added to database for first time")
        return
    
    def get_variant(self, variant):
        """Check if a variant exists in the database and return it
    
            Search the variants with the variant id
        
            Args:
                variant (dict): A variant dictionary
        
            Returns:
                variant (dict): A variant dictionary
        """
        return self.db.variant.find_one({'_id': variant.get('_id')})

    def get_variants(self):
        """Return all variants in the database
    
            Returns:
                variants (Iterable(Variant))
        """
        return self.db.variant.find()

    def delete_variant(self, variant):
        """Remove variant from database
            
            This means that we take down the observations variable with one.
            If 'observations' == 1 we remove the variant. If variant was homozygote
            we decrease 'homozygote' with one.
            
            Args:
                variant (dict): A variant dictionary            
        """
        mongo_variant = self.get_variant(variant)
        
        if mongo_variant:
            
            if mongo_variant['observations'] == 1:
                logger.debug("Removing variant {0}".format(
                    mongo_variant.get('_id')
                ))
                message = self.db.variant.remove({'_id': variant['_id']})
            else:
                logger.debug("Decreasing observations for {0}".format(
                    mongo_variant.get('_id')
                ))
                message = self.db.variant.update({
                    '_id': mongo_variant['_id']
                    },{
                        '$inc': {
                            'observations': -1,
                            'homozygote': -(variant.get('homozygote', 0))
                        }
                    }, upsert=False)
        return
        
    def add_bulk(self, variants):
        """Insert a bulk of variants
        
            Args:
                variants(Iterable(dict)) : A iterable with variants
        
        """
        
        bulk = self.db.variant.initialize_ordered_bulk_op()
        for variant in variants:
            if variant:
                bulk.find({'_id': variant['_id']}).upsert().update(
                    {
                        '$inc': {
                            'homozygote': variant.get('homozygote', 0),
                            'observations': 1
                        }
                     }
                )
        message = bulk.execute()
        logger.debug("Number of variants inserted: {0}".format(
            message.get('nInserted')
        ))
        logger.debug("Number of variants upserted: {0}".format(
            message.get('nUpserted')
        ))
        return
    
    


# -*- coding: utf-8 -*-
from datetime import datetime
import logging

from loqusdb.vcf_tools import get_formated_variant

logger = logging.getLogger(__name__)


def load_variants(adapter, family_id, affected_individuals, variant_stream,
                  bulk_insert=False, vcf_path=None):
    """Load variants for a family into the database.

    Args:
        adapter (loqusdb.plugins.MongoAdapter): initialized plugin
        family_id (str): unique family identifier
        affected_inidividuals (List[str]): list to match individuals
        variant_stream (sequence): stream of VCF lines
        bulk_insert (bool): whether to insert in bulk or one-by-one
        vcf_path (path): for storing in database
    """
    case = {'case_id': family_id, 'vcf_path': vcf_path}
    adapter.add_case(case)

    # This is the header line with mandatory vcf fields
    header = []
    nr_of_variants = 0
    nr_of_inserted = 0

    start_inserting = datetime.now()
    start_ten_thousand = datetime.now()

    variants = []
    for line in variant_stream:
        line = line.rstrip()
        if line.startswith('#'):
            if not line.startswith('##'):
                header = line[1:].split()
        else:
            nr_of_variants += 1

            formated_variant = get_formated_variant(
                variant_line=line, header_line=header,
                affected_individuals=affected_individuals)

            if formated_variant:
                nr_of_inserted += 1
                if bulk_insert:
                    variants.append(formated_variant)
                else:
                    adapter.add_variant(variant=formated_variant)

            if nr_of_variants % 10000 == 0:
                logger.info("{0} of variants processed".format(nr_of_variants))
                logger.info("Time to insert last 10000: {0}".format(
                    datetime.now()-start_ten_thousand))
                start_ten_thousand = datetime.now()

            if nr_of_variants % 100000 == 0:
                if bulk_insert:
                    adapter.add_bulk(variants)
                    variants = []

    if bulk_insert:
        adapter.add_bulk(variants)

    logger.info("Nr of variants in vcf: {0}".format(nr_of_variants))
    logger.info("Nr of variants inserted: {0}".format(nr_of_inserted))
    logger.info("Time to insert variants: {0}".format(datetime.now() -
                                                      start_inserting))



# -*- coding: utf-8 -*-
import logging

from loqusdb.vcf_tools import get_formated_variant

logger = logging.getLogger(__name__)


def delete_variants(adapter, variant_stream, family_id, affected_individuals):
    case = {'case_id': family_id}
    adapter.delete_case(case)

    header = []
    nr_of_deleted = 0
    for line in variant_stream:
        line = line.rstrip()
        if line.startswith('#'):
            if not line.startswith('##'):
                header = line[1:].split()
        else:
            formated_variant = get_formated_variant(
                variant_line=line, header_line=header,
                affected_individuals=affected_individuals)

            adapter.delete_variant(formated_variant)
            nr_of_deleted += 1

    return nr_of_deleted



# -*- coding: utf-8 -*-
import logging
import click
from datetime import datetime
import sys

from vcftoolbox import get_vcf_handle

from loqusdb.utils import get_family
from loqusdb.exceptions import CaseError
from loqusdb.utils import delete_variants
from . import base_command

logger = logging.getLogger(__name__)

@base_command.command()
@click.argument('variant_file',
                    nargs=1,
                    type=click.Path(exists=True),
                    metavar='<vcf_file> or -'
)
@click.option('-f', '--family_file',
                    nargs=1, 
                    type=click.File('r'),
                    metavar='<ped_file>'
)
@click.option('-t' ,'--family_type', 
                type=click.Choice(['ped', 'alt', 'cmms', 'mip']), 
                default='ped',
                help='If the analysis use one of the known setups, please specify which one.'
)
@click.option('-i', '--family_id',
                    nargs=1, 
                    type=str,
                    help='The id for the case to add'
)
@click.pass_context
def delete(ctx, variant_file, family_file, family_type, family_id):
    """Delete the variants of a case."""
    if not family_file or family_id:
        logger.error("Please provide a family file or a case id")
        logger.info("Exiting")
        ctx.abort()

    family = get_family(
        family_lines=family_file,
        family_type=family_type
    )

    family_id = family.family_id
    affected_individuals = family.affected_individuals
    adapter = ctx.obj['adapter']

    if variant_file == '-':
        logger.info("Start parsing variants from stdin")
        variant_stream = get_vcf_handle(fsock=sys.stdin)
    else:
        logger.info("Start parsing variants from stdin")
        variant_stream = get_vcf_handle(infile=variant_file)

    start_deleting = datetime.now()
    try:
        count = delete_variants(adapter, variant_stream, family_id,
                                affected_individuals)
    except CaseError as error:
        logger.warning(error.message)
        ctx.abort()

    logger.info("Nr of variants deleted: {0}".format(count))
    logger.info("Time to delete variants: {0}"
                .format(datetime.now() - start_deleting))
